## second step of risk analysis, with the following steps:
## - calculate socio-economic impacts including socio-economic projections 
## - perform calculation for adaptation scenarios

## NOTE: all parte related to calculation of exposed built-up area are commented but 
##      they can be reactivated if necessary

#### SET RUN MODE
run<-"cluster" ## choose whether to run on (local) or (cluster)
adst<-"unif" ## type of adaptation strategy: uniform (unif) or variable (var) return period
ensemble <-"mean" ## choose whether to use ensemble discharges calculated with median or mean (only for stabilized climate)
flag_MS <- "sta" ## use model runs as transient simulations "tra" (i.e. original data) or stabilized climate "sta"
## stabilized "sta" (data after year of hitting WLs is replaced with constant)
## not used here
flag_prj<-"REF" ## how to consider socio-economic scenarios: "REF" use two reference JRC periods (2050-2010) "JRC"-"SSP": variable JRC or SSP projections

## load libraries
library(sp,lib.loc="/home/dottofr/Rlib")     	# needed to load rdgdal library
Sys.sleep(1)
library(raster,lib.loc="/home/dottofr/Rlib")
Sys.sleep(1)
library(rgdal,lib.loc="/home/dottofr/Rlib") 	# use to read .map raster format
#### 

if(run=='cluster'){
  args <- commandArgs(TRUE)
  modelRun <- as.character(args[1])     # model run to analyse
  mainDir <- as.character(args[2])     # directory of config file   
} else if(run=='local'){
  modelRun <- "projection_dis_rcp85_CLMcom-CCLM4-8-17_BC_CNRM-CERFACS-CNRM-CM5_wuConst_statistics_ptId.nc" 
}
#### SET PARAMETERS USED FOR THE SENSISTIVITY ANALYSIS (for sensitivity analysis)
flag_WL <- as.character(args[3]) # stabilized WL scenarios to use (1.5C 2C 3C) if flag_MS = "sta" 
prot_par<- as.character(args[4]) # level of flood protection - 050: 50% of estimated average protection standards
ead_int<-c(-0.29,0.49) # 90% confidence interval to quantify the uncertainty in damage functions
ead_qnt<-c("q05","q50","q95") # quantiles to use to quantify the uncertainty in damage functions 
# flag_WL <- "3C" 
# prot_par<- "050"

workDir<-(paste(mainDir,"/PESETA_IV_data",sep=""))

#############################
print('---')
print('PERFORM RISK ANALYSIS FOR PRESENT AND FUTURE SCENARIOS')
print('---')
#############################
# num_ep <-dim(dis_f)[1] # number of EFAS network cells 
num_ep <- 80553

# initialize RP classes considered and probabilities for integrating risk curve
RP_classes<- c(2,10,20,50,100,200,500,1000)
# initialize reference years
RY_classes<- c(1981,1985,1990,1995,2000,2005,2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
RY_classes_prj<- c(2010,2015,2020,2025,2030,2035,2040,2045,2050,2055,2060,2065,2070,2075,2080,2085,2090,2095,2100)
nRP<-length(RP_classes)
nRY<-length(RY_classes)

#########
## READ INPUT TABLES

## read table of model runs and warming levels
WL_table <- read.csv(paste(workDir,"/tables/table_WL_ModelRuns.csv", sep=""))
lMod<-dim(WL_table)[1]

## read NUTS2 and country tables
table_EP_nuts <- read.csv(paste(workDir,"/tables/table_EP_NUTS2.csv", sep=""))
table_countries <- read.csv(paste(workDir,"/tables/table_nuts_countries.csv", sep=""))
## read table of XY coordinates for EFAS points
table_EP_XY <- read.csv(paste(workDir,"/tables/table_EP_coordinates.csv", sep=""))
## NOTE: Alternative calculation of NUTS2 grid
table_EP_nuts_XY<-cbind(table_EP_nuts,table_EP_XY)

#########
## FORMAT AND INITIALIZE  DATA

## format NUTS2-country tables
list_nuts2<-unique(table_countries[,2])
nNU<-length(list_nuts2)

## read model run set up and corresponding warming levels
ind_MR<- which(WL_table[,"ModelRun"]==modelRun)
Pr_mod<-WL_table[ind_MR,"Model"]
Pr_rcp<-WL_table[ind_MR,"RCP"]
Pr_wu<-WL_table[ind_MR,"WU"]

## we consider only hazard reduction adaptation scenarios
nSC<-16
SC_classes<-c(2,10,15,20,30,50,70,100,150,200,350,500,700,1000,1500,2000)
 iad1=1 
# iad1=4  # we don't consider return periods below 20 years

print('done reading input data')
#############################
print('---')
print('UPDATE IMPACTS ACCOUNTING FOR SOCIO-ECONOMIC PROJECTIONS')
print('---')

#####################
print("- calculate impacts from future socio-economic projections")
## elaborate all models together

table_gdp_jrc<-read.csv(paste(workDir,"/tables/projections/table_nuts_gdp_PIV.csv", sep=""))
table_pop_jrc<-read.csv(paste(workDir,"/tables/projections/table_nuts_pop_PIII.csv", sep=""))    
coef_GDP <- 0.3558   ## USE SAME COEFFICIENT 0.3558 for all sectors

table_gdp_ref<- array(0, dim=c(nNU,nRY-6) ,dimnames = list(list_nuts2,RY_classes_prj))
table_pop_ref<- array(0, dim=c(nNU,nRY-6) ,dimnames = list(list_nuts2,RY_classes_prj))
table_gdp_ref<- table_gdp_ref+table_gdp_jrc[,"X2015"]
table_gdp_ratio <- (table_gdp_jrc[,3:21]^coef_GDP)/(table_gdp_ref^coef_GDP) 
## NOTE: for PESETA, we use population after 2050, years up to 2050 are covered by spatial maps
table_pop_ref<- table_pop_ref+table_pop_jrc[,"X2050"]
table_pop_ratio  <- table_pop_jrc[,3:21]/table_pop_ref
table_pop_ratio[,1:8]  <- 1

  
##  CALCULATE NO-ADAPTATION SCENARIOS (used also for relocation and f.p. measures!)   
print("CALCULATE NO-ADAPTATION SCENARIOS")     

  if(flag_MS=="sta"){
    fileSuf<- paste0(flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
    ead_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_",ensemble,"_fp",prot_par,"/ead_ep_",fileSuf),row.name=1) 
    eapa_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf),row.name=1) 
#    eab_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/eab_ep_adapt_",ensemble,"_fp",prot_par,"/eab_ep_",fileSuf),row.name=1)    
  } else if (flag_MS=="tra"){ 
    ead_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_ep_clim_fp",prot_par,"/ead_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1) 
    eapa_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_ep_clim_fp",prot_par,"/eapa_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1) 
#    eab_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/eab_ep_clim_fp",prot_par,"/eab_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1) 
  }
  ## initialize arrays with values corrected with projection  
  ead_f_prj <- ead_f
  eapa_f_prj <- eapa_f 
#  eab_f_prj <- eab_f   
  
#  for(i in seq(7,nRY,by=2)){  # start from 2010 and then every 10 years (years with projection)
  for(i in 7:nRY){  # start from 2010 and then every 5 years (years with projection)
    ## create and write raster file of EP results      
    ead_fs<-cbind(table_EP_XY[,2:3],ead_f[,i])
    ead_grid<- rasterFromXYZ(ead_fs, res=c(5000,5000), crs=NA, digits=5)
    eapa_fs<-cbind(table_EP_XY[,2:3],eapa_f[,i])
    eapa_grid<- rasterFromXYZ(eapa_fs, res=c(5000,5000), crs=NA, digits=5)
#    eab_fs<-cbind(table_EP_XY[,2:3],eab_f[,i])
#    eab_grid<- rasterFromXYZ(eab_fs, res=c(5000,5000), crs=NA, digits=5)
    ## read population and urban projections (give as ratio to present values) and scale future impacts  
    
    popMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_JRC_projections/Population/Qi_5km_",RY_classes[i],"_change.tif"),silent=TRUE)) 
    urbMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_JRC_projections/Urban/HA_Urban_5km_",RY_classes[i],"_change.tif"),silent=TRUE))
    popMap<-(popMap+100)/100            
    urbMap<-(urbMap+100)/100
    urbMap[is.na(urbMap)]<-0
    popMap[is.na(popMap)]<-0      
    popMap[popMap==0]<-1 # to avoid eliminating cells with values
    urbMap[urbMap==0]<-1
    
## include this part to make sure that maps return unchanged with no GDP-pop-urb changes
#    popMap[popMap]<-1 # assign 1 everywhere
#    urbMap[urbMap]<-1
  
    popClip<-popMap*eapa_grid
    damClip<-urbMap*ead_grid  
#    bldClip<-urbMap*eab_grid    
    
    ead_prj <- rasterToPoints(damClip)      
    eapa_prj <- rasterToPoints(popClip)            
#    eab_prj <- rasterToPoints(bldClip)          
    ead_f_prj[,i] <- ead_prj[,3]
    eapa_f_prj[,i] <- eapa_prj[,3]  
#    eab_f_prj[,i] <- eab_prj[,3]
    
    rm(ead_fs,eapa_fs,eab_fs,ead_grid,eapa_grid,eab_grid,popMap,damMap,popClip,damClip,bldClip)
  }  
  ## dam tables to be  multiplied with GDP and coefficients
  ## pop tables after 2050 to be multiplied with country-scale projections    
  gdp_ep <- array(1, dim = c(num_ep, nRY), dimnames = list(c(1:num_ep),RY_classes))
  pop_ep <- array(1, dim = c(num_ep, nRY), dimnames = list(c(1:num_ep),RY_classes))

## comment this part to make sure that maps return unchanged with no GDP-pop-urb changes
 for (i in 1:nNU){
   ind1<-which(table_EP_nuts[,2]==table_gdp_jrc[i,"NUTS_CODE"])
   ind2<-which(table_EP_nuts[,2]==table_pop_jrc[i,"NUTS_CODE"])
   for (j in 1:dim(table_gdp_ratio)[2]){
     gdp_ep[ind1,(j+6)]<-table_gdp_ratio[i,j]
     pop_ep[ind2,(j+6)]<-table_pop_ratio[i,j]       
   }
 }
  ead_f_prj <- ead_f_prj*gdp_ep
  eapa_f_prj <- eapa_f_prj*pop_ep

## calculate results accounting for the uncertainty in damage functions 
##  (we use a confidence interval given by q1-q3 quantiles)    

ead_f_prj_q1 <- ead_f_prj*(1+ead_int[1])
ead_f_prj_q3 <- ead_f_prj*(1+ead_int[2])

## TEST: AGGREGATE AT NUTS2 LEVEL TO COMPARE WITH 05B
## NOTE: this script provides correct result
# print("aggregate expected annual impacts at NUTS2 scale (step 5F")
# 
# ead_f_nuts2 <- array(0, dim = c(nNU,nRY), dimnames=list(list_nuts2,RY_classes))
# eapa_f_nuts2 <- array(0, dim = c(nNU,nRY), dimnames=list(list_nuts2,RY_classes))
#    for (ny in 1:nRY){ 
#      for (np in 1:nNU) {
#    #    print(paste("NUTS2 ",list_nuts2[np],sep=""))
#        ind_nuts<-which(table_EP_nuts[,2]==list_nuts2[np])
#        ead_f_nuts2[np,ny]<-sum(ead_f_prj[ind_nuts,ny]) 
#        eapa_f_nuts2[np,ny]<-sum(eapa_f_prj[ind_nuts,ny]) 
#      }
#    }
# fileSuf<- paste0(flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,"_no_prj2.csv")
# write.csv(ead_f_nuts2,file = paste0(workDir,"/tables/EAI_Europe/test_06b/ead_nuts2_",fileSuf),row.names=TRUE)
# write.csv(eapa_f_nuts2,file = paste0(workDir,"/tables/EAI_Europe/test_06b/eapa_nuts2_",fileSuf),row.names=TRUE)
## TEST

  print("print and save results")
  if(flag_MS=="sta"){
    dir.create( paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",ead_qnt[1]))
    dir.create( paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",ead_qnt[2]))
    dir.create( paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",ead_qnt[3]))    
    dir.create( paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_PIV_",ensemble,"_fp",prot_par))   
    # dir.create( paste0(workDir,"/tables/EAI_Europe/eab_ep_adapt_PIV_",ensemble,"_fp",prot_par)) 
    
  
    fileSuf<- paste0(flag_WL,"_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
    write.csv(ead_f_prj,file   =  paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",ead_qnt[2],"/ead_ep_",fileSuf),row.names=TRUE)
    write.csv(ead_f_prj_q1,file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",ead_qnt[1],"/ead_ep_",fileSuf),row.names=TRUE)
    write.csv(ead_f_prj_q3,file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",ead_qnt[3],"/ead_ep_",fileSuf),row.names=TRUE)      
    write.csv(eapa_f_prj,file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf),row.names=TRUE)
#    write.csv(eab_f_prj,file = paste0(workDir,"/tables/EAI_Europe/eab_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eab_ep_",fileSuf),row.names=TRUE)    
  } else if(flag_MS=="tra"){
    fileSuf<- paste0("_transient_na_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
    write.csv(ead_f_prj,file = paste0(workDir,"/tables/EAI_Europe/ead_ep_transient_PIV_fp",prot_par,"/ead_ep_",fileSuf),row.names=TRUE)
    write.csv(eapa_f_prj,file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_transient_PIV_fp",prot_par,"/eapa_ep_",fileSuf),row.names=TRUE)
#    write.csv(eab_f_prj,file = paste0(workDir,"/tables/EAI_Europe/eab_ep_transient_PIV_fp",prot_par,"/eab_ep_",fileSuf),row.names=TRUE)
  }
  
  print("---")
################# 
##  LOOP OVER ALL ADAPTATION SCENARIOS
  print("CALCULATE ADAPTATION SCENARIOS") 
  
  for(iad in iad1:nSC){
#  for(iad in iad1:2){    
    
    print(paste0("adaptation scenario ",iad," of ",length(SC_classes)," -  RP ", SC_classes[iad], " years"))  
    if(flag_MS=="sta"){
      fileSuf<- paste0(flag_WL,"_",adst,"_RP",SC_classes[iad],"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
      ead_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_",ensemble,"_fp",prot_par,"/ead_ep_",fileSuf),row.name=1) 
      eapa_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf),row.name=1)  
    } else if (flag_MS=="tra"){ 
      ead_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/ead_ep_transient_fp",prot_par,"/ead_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1) 
      eapa_f <- read.csv(paste0(workDir,"/tables/EAI_Europe/eapa_ep_transient_fp",prot_par,"/eapa_ep_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv"),row.name=1) 
    }
## initialize arrays with values corrected with projection  
    ead_f_prj <- ead_f
    eapa_f_prj <- eapa_f 
  
  #  for(i in seq(7,nRY,by=2)){  # start from 2015 and then every 10 years (years with projection)
    for(i in 7:nRY){  # start from 2015 and then every 5 years (years with projection)
      ## create and write raster file of EP results      
      ead_fs<-cbind(table_EP_XY[,2:3],ead_f[,i])
      ead_grid<- rasterFromXYZ(ead_fs, res=c(5000,5000), crs=NA, digits=5)
      eapa_fs<-cbind(table_EP_XY[,2:3],eapa_f[,i])
      eapa_grid<- rasterFromXYZ(eapa_fs, res=c(5000,5000), crs=NA, digits=5)
  ## read population and urban projections (give as ratio to present values) and scale future impacts  
     
      popMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_JRC_projections/Population/Qi_5km_",RY_classes[i],"_change.tif"),silent=TRUE)) 
      urbMap<-raster(readGDAL(paste0(workDir,"/maps/PIV_JRC_projections/Urban/HA_Urban_5km_",RY_classes[i],"_change.tif"),silent=TRUE))
      popMap<-(popMap+100)/100            
      urbMap<-(urbMap+100)/100
      urbMap[is.na(urbMap)]<-0
      popMap[is.na(popMap)]<-0    
      popMap[popMap==0]<-1 # to avoid eliminating cells with values
      urbMap[urbMap==0]<-1      
      popClip<-popMap*eapa_grid
      damClip<-urbMap*ead_grid       

      ead_prj <- rasterToPoints(damClip)      
      eapa_prj <- rasterToPoints(popClip)            
      ead_f_prj[,i] <- ead_prj[,3]
      eapa_f_prj[,i] <- eapa_prj[,3]  
      
      rm(ead_fs,eapa_fs,ead_grid,eapa_grid,popMap,urbMap,popClip,damClip)
    }  
 
## dam tables to be  multiplied with GDP and coefficients
## pop tables after 2050 to be multiplied with country-scale projections    
    gdp_ep <- array(1, dim = c(num_ep, nRY), dimnames = list(c(1:num_ep),RY_classes))
    pop_ep <- array(1, dim = c(num_ep, nRY), dimnames = list(c(1:num_ep),RY_classes))
    for (i in 1:nNU){
      ind1<-which(table_EP_nuts[,2]==table_gdp_jrc[i,"NUTS_CODE"])
      ind2<-which(table_EP_nuts[,2]==table_pop_jrc[i,"NUTS_CODE"])
      for (j in 1:dim(table_gdp_ratio)[2]){
       gdp_ep[ind1,(j+6)]<-table_gdp_ratio[i,j]
       pop_ep[ind2,(j+6)]<-table_pop_ratio[i,j]       
      }
    }
    ead_f_prj <- ead_f_prj*gdp_ep
    eapa_f_prj <- eapa_f_prj*pop_ep

## calculate results accounting for the uncertainty in damage functions 
##  (we use a 90% confidence interval)    
    ead_f_prj_q1 <- ead_f_prj*gdp_ep*(1+ead_int[1])
    ead_f_prj_q3 <- ead_f_prj*gdp_ep*(1+ead_int[2])
    
print("print and save results")
    if(flag_MS=="sta"){
      fileSuf<- paste0(flag_WL,"_",adst,"_RP",SC_classes[iad],"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
      write.csv(ead_f_prj,file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",ead_qnt[2],"/ead_ep_",fileSuf),row.names=TRUE)
      write.csv(ead_f_prj_q1,file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",ead_qnt[1],"/ead_ep_",fileSuf),row.names=TRUE)
      write.csv(ead_f_prj_q3,file = paste0(workDir,"/tables/EAI_Europe/ead_ep_adapt_PIV_",ensemble,"_fp",prot_par,"_ed",ead_qnt[3],"/ead_ep_",fileSuf),row.names=TRUE)      
      write.csv(eapa_f_prj,file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_adapt_PIV_",ensemble,"_fp",prot_par,"/eapa_ep_",fileSuf),row.names=TRUE)
    } else if(flag_MS=="tra"){
      fileSuf<- paste0("_transient_",adst,"_RP",SC_classes[iad],"_",Pr_rcp,"_",Pr_mod,"_wu",Pr_wu,".csv")
      write.csv(ead_f_prj,file = paste0(workDir,"/tables/EAI_Europe/ead_ep_transient_PIV_fp",prot_par,"/ead_ep_",fileSuf),row.names=TRUE)
      write.csv(eapa_f_prj,file = paste0(workDir,"/tables/EAI_Europe/eapa_ep_transient_PIV_fp",prot_par,"/eapa_ep_",fileSuf),row.names=TRUE)
    }
  }

#####################
print('risk analysis COMPLETED')
print('script ended')

